var __extends = (this && this.__extends) || function (d, b) {
    for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p];
    function __() { this.constructor = d; }
    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
};
var DomWalker = require('./DomWalker');
var commonmark = require('commonmark');
var Util = require('./Util');
var DomUtil = require('./DomUtil');
var MarkdownUtil = require('./MarkdownUtil');
var Converter = (function () {
    function Converter(parser, options) {
        this.parser = parser;
        this.options = Util.assign({
            rawHtmlElements: ['div', 'table', 'td', 'tr', 'th', 'tbody', 'thead'],
            ignoredHtmlElements: [],
            interpretUnknownHtml: true
        }, options);
    }
    Converter.prototype.convert = function (html) {
        html = html.trim();
        var htmlElement = this.parser.parse(html);
        var walker = new DomWalker(htmlElement);
        var document = new commonmark.Node('Document');
        var conversion = this.createConversion(walker.next().domNode, walker).execute(document);
        return document;
    };
    Converter.prototype.createConversion = function (domNode, walker) {
        var nodeName = domNode.nodeName.toLowerCase();
        if (this.options.rawHtmlElements.indexOf(nodeName) >= 0) {
            return new RawHtmlConversion(walker, this, domNode);
        }
        if (this.options.ignoredHtmlElements.indexOf(nodeName) >= 0) {
            return new NoopConversion(walker, this);
        }
        switch (nodeName) {
            case 'a':
                return new LinkConversion(walker, this, domNode);
            case 'br':
                return new NamedContainerConversion(walker, this, 'Hardbreak');
            case 'body':
            case 'custom-root':
                return new NoopConversion(walker, this);
            case 'pre':
                return new NamedContainerConversion(walker, this, 'CodeBlock');
            case 'code':
                return new CodeBlockConversion(walker, this, domNode);
            case 'img':
                return new ImageConversion(walker, this, domNode);
            case 'ul':
            case 'ol':
                return new ListConversion(walker, this, domNode);
            case 'li':
                return new ListItemConversion(walker, this);
            case 'p':
                return new NamedContainerConversion(walker, this, 'Paragraph');
            case 'hr':
                return new NamedContainerConversion(walker, this, 'HorizontalRule');
            case '#text':
                return new TextConversion(walker, this, domNode);
            case 'blockquote':
                return new NamedContainerConversion(walker, this, 'BlockQuote');
            case 'i':
            case 'em':
                return new InlineConversion(walker, this, 'Emph');
            case 'b':
            case 'strong':
                return new InlineConversion(walker, this, 'Strong');
            case 'h1':
            case 'h2':
            case 'h3':
            case 'h4':
            case 'h5':
            case 'h6':
            case 'h7':
            case 'h8':
            case 'h9':
                return new HeaderConversion(walker, this, parseInt(domNode.nodeName.substr(1)));
            default:
                if (this.options.interpretUnknownHtml) {
                    return new RawHtmlConversion(walker, this, domNode);
                }
                else {
                    return new NoopConversion(walker, this);
                }
        }
    };
    return Converter;
})();
var NodeConversion = (function () {
    function NodeConversion(domWalker, converter) {
        this.children = [];
        var next;
        while ((next = domWalker.next()).isEntering) {
            this.children.push(converter.createConversion(next.domNode, domWalker));
        }
    }
    return NodeConversion;
})();
var NoopConversion = (function (_super) {
    __extends(NoopConversion, _super);
    function NoopConversion() {
        _super.apply(this, arguments);
    }
    NoopConversion.prototype.execute = function (container) {
        return this.children.map(function (c) { return c.execute(container); }).pop();
    };
    return NoopConversion;
})(NodeConversion);
var NamedContainerConversion = (function (_super) {
    __extends(NamedContainerConversion, _super);
    function NamedContainerConversion(domWalker, converter, nodeName, literal) {
        if (literal === void 0) { literal = null; }
        _super.call(this, domWalker, converter);
        this.nodeName = nodeName;
        this.literal = literal;
    }
    NamedContainerConversion.prototype.execute = function (container) {
        var node = new commonmark.Node(this.nodeName);
        node.literal = this.literal;
        if (container) {
            container.appendChild(node);
        }
        this.children.forEach(function (c) { return c.execute(node); });
        return node;
    };
    return NamedContainerConversion;
})(NodeConversion);
var ContainerBlockConversion = (function (_super) {
    __extends(ContainerBlockConversion, _super);
    function ContainerBlockConversion(domWalker, converter, nodeName) {
        _super.call(this, domWalker, converter, nodeName);
    }
    ContainerBlockConversion.prototype.execute = function (container) {
        var containerBlockNode = _super.prototype.execute.call(this, container);
        var wrapInlineNodes = function (inlineNodes) {
            if (inlineNodes.length) {
                var wrapper = new commonmark.Node('Paragraph');
                inlineNodes[0].insertBefore(wrapper);
                inlineNodes.forEach(function (c) { return wrapper.appendChild(c); });
            }
        };
        if (containerBlockNode.firstChild) {
            var child = containerBlockNode.firstChild;
            var inlineNodes = [];
            while (child) {
                if (MarkdownUtil.isInline(child)) {
                    inlineNodes.push(child);
                }
                else {
                    wrapInlineNodes(inlineNodes);
                    inlineNodes = [];
                }
                child = child.next;
            }
            wrapInlineNodes(inlineNodes);
        }
        return containerBlockNode;
    };
    return ContainerBlockConversion;
})(NamedContainerConversion);
var LinkConversion = (function (_super) {
    __extends(LinkConversion, _super);
    function LinkConversion(domWalker, converter, anchorTag) {
        _super.call(this, domWalker, converter, 'Link');
        this.anchorTag = anchorTag;
    }
    LinkConversion.prototype.execute = function (container) {
        var link = _super.prototype.execute.call(this, container);
        var href = this.anchorTag.attributes.getNamedItem('href');
        var title = this.anchorTag.attributes.getNamedItem('title');
        if (href) {
            link.destination = href.value;
        }
        else {
            link.destination = '';
        }
        if (title) {
            link.title = title.value;
        }
        else {
            link.title = '';
        }
        return link;
    };
    return LinkConversion;
})(NamedContainerConversion);
var HeaderConversion = (function (_super) {
    __extends(HeaderConversion, _super);
    function HeaderConversion(domWalker, converter, level) {
        _super.call(this, domWalker, converter, 'Header');
        this.level = level;
    }
    HeaderConversion.prototype.execute = function (container) {
        var headerNode = _super.prototype.execute.call(this, container);
        headerNode.level = this.level;
        return headerNode;
    };
    return HeaderConversion;
})(NamedContainerConversion);
var InlineConversion = (function (_super) {
    __extends(InlineConversion, _super);
    function InlineConversion(domWalker, converter, nodeName) {
        _super.call(this, domWalker, converter, nodeName);
    }
    return InlineConversion;
})(NamedContainerConversion);
var TextConversion = (function (_super) {
    __extends(TextConversion, _super);
    function TextConversion(domWalker, converter, textNode) {
        _super.call(this, domWalker, converter);
        this.textNode = textNode;
    }
    TextConversion.prototype.execute = function (container) {
        var textContent = this.trimTextContent();
        if (this.hasParent('pre')) {
            container.literal = this.textNode.textContent;
            return null;
        }
        else if (textContent) {
            var lines = textContent.split(TextConversion.SOFTBREAK_SUBSTITUTION_CHARACTER);
            lines.forEach(function (line, index) {
                if (line) {
                    var node = new commonmark.Node('Text');
                    node.literal = line;
                    container.appendChild(node);
                }
                if (lines.length > 1 && index != (lines.length - 1)) {
                    container.appendChild(new commonmark.Node('Softbreak'));
                }
            });
            return container.firstChild;
        }
    };
    TextConversion.prototype.hasParent = function (type) {
        var parent = this.textNode;
        while (parent = parent.parentNode) {
            if (parent.nodeName.toLowerCase() === 'code') {
                return true;
            }
        }
        return false;
    };
    TextConversion.prototype.trimTextContent = function () {
        var text = this.textNode.textContent;
        if (this.shouldTrimSpaces(this.textNode.previousSibling)) {
            text = this.trimLeft(text);
        }
        if (this.shouldTrimSpaces(this.textNode.nextSibling)) {
            text = this.trimRight(text);
        }
        return text;
    };
    TextConversion.prototype.shouldTrimSpaces = function (sibling) {
        return !sibling || !DomUtil.isInline(sibling) || TextConversion.INLINE_HTML_NODES_OF_WHICH_SIBLINGS_SHOULD_BE_TRIMMED.indexOf(sibling.nodeName.toLowerCase()) >= 0;
    };
    TextConversion.prototype.trimLeft = function (text) {
        while (text.charAt(0).match(/\s/)) {
            text = text.substr(1);
        }
        return text;
    };
    TextConversion.prototype.trimRight = function (text) {
        while (text.charAt(text.length - 1).match(/\s/)) {
            text = text.substr(0, text.length - 1);
        }
        return text;
    };
    TextConversion.INLINE_HTML_NODES_OF_WHICH_SIBLINGS_SHOULD_BE_TRIMMED = ['br', 'h2', 'h3', 'h4', 'h5', 'h6', 'h7', 'h8', 'h9'];
    TextConversion.SOFTBREAK_SUBSTITUTION_CHARACTER = '\n';
    return TextConversion;
})(NodeConversion);
var ImageConversion = (function (_super) {
    __extends(ImageConversion, _super);
    function ImageConversion(domWalker, converter, imgTag) {
        _super.call(this, domWalker, converter, 'Image');
        this.imgTag = imgTag;
    }
    ImageConversion.prototype.execute = function (container) {
        var imageNode = _super.prototype.execute.call(this, container);
        var href = this.imgTag.attributes.getNamedItem('src');
        var title = this.imgTag.attributes.getNamedItem('title');
        var alternative = this.imgTag.attributes.getNamedItem('alt');
        if (alternative) {
            var altTextNode = new commonmark.Node('Text');
            imageNode.appendChild(altTextNode);
            altTextNode.literal = alternative.value;
        }
        if (href) {
            imageNode.destination = href.value;
        }
        else {
            imageNode.destination = '';
        }
        if (title) {
            imageNode.title = title.value;
        }
        else {
            imageNode.title = '';
        }
        return imageNode;
    };
    return ImageConversion;
})(NamedContainerConversion);
var ListConversion = (function (_super) {
    __extends(ListConversion, _super);
    function ListConversion(domWalker, converter, listTag) {
        _super.call(this, domWalker, converter, 'List');
        this.listTag = listTag;
    }
    ListConversion.prototype.execute = function (container) {
        var list = _super.prototype.execute.call(this, container);
        list._listData = {};
        var start = this.listTag.attributes.getNamedItem('start');
        switch (this.listTag.nodeName.toLowerCase()) {
            case 'ul':
                list.listType = 'Bullet';
                break;
            case 'ol':
                list.listType = 'Ordered';
                if (start && start.value) {
                    list.listStart = parseInt(start.value);
                }
                break;
        }
        return list;
    };
    return ListConversion;
})(NamedContainerConversion);
var ListItemConversion = (function (_super) {
    __extends(ListItemConversion, _super);
    function ListItemConversion(domWalker, converter) {
        _super.call(this, domWalker, converter, 'Item');
    }
    return ListItemConversion;
})(ContainerBlockConversion);
var CodeBlockConversion = (function (_super) {
    __extends(CodeBlockConversion, _super);
    function CodeBlockConversion(domWalker, converter, codeTag) {
        _super.call(this, domWalker, converter);
        this.codeTag = codeTag;
    }
    CodeBlockConversion.prototype.isInline = function () {
        return false;
    };
    CodeBlockConversion.prototype.execute = function (container) {
        var codeBlock = null;
        if (container.type === 'CodeBlock') {
            this.enrichCodeBlock(this.codeTag, container);
        }
        else {
            codeBlock = new commonmark.Node('Code');
            container.appendChild(codeBlock);
            codeBlock.literal = '';
            this.enrichCodeBlock(this.codeTag, codeBlock);
        }
        var parent = codeBlock;
        if (!parent) {
            parent = container;
        }
        this.children.forEach(function (c) { return c.execute(parent); });
        return codeBlock;
    };
    CodeBlockConversion.prototype.enrichCodeBlock = function (codeTag, codeBlock) {
        if (codeBlock.type === 'CodeBlock' && DomUtil.isElement(codeTag)) {
            var classes = codeTag.classList;
            var info = null;
            for (var i = 0; i < classes.length; i++) {
                var clazz = classes.item(i);
                if (clazz.substr(0, 9) === 'language-') {
                    info = clazz.substr(9);
                }
            }
            codeBlock.info = info;
            codeBlock.literal = '';
        }
    };
    return CodeBlockConversion;
})(NodeConversion);
var RawHtmlConversion = (function (_super) {
    __extends(RawHtmlConversion, _super);
    function RawHtmlConversion(domWalker, converter, rawHtmlNode) {
        _super.call(this, domWalker, converter);
        this.rawHtmlNode = rawHtmlNode;
    }
    RawHtmlConversion.prototype.isInline = function () {
        return false;
    };
    RawHtmlConversion.prototype.execute = function (container) {
        var rawHtmlNode = this.rawHtmlNode;
        if (DomUtil.isElement(rawHtmlNode)) {
            var nodeName = 'HtmlBlock';
            if (this.trueForAllChildren(DomUtil.isText)) {
                return this.appendChildNode(container, nodeName, rawHtmlNode.outerHTML);
            }
            else {
                this.appendChildNode(container, nodeName, DomUtil.writeStartElement(rawHtmlNode));
                this.children.forEach(function (c) { return c.execute(container); });
                return this.appendChildNode(container, nodeName, DomUtil.writeEndElement(rawHtmlNode));
            }
        }
        else if (DomUtil.isComment(rawHtmlNode)) {
            return this.appendChildNode(container, 'HtmlBlock', '<!--' + rawHtmlNode.data + '-->');
        }
        else if (DomUtil.isText(rawHtmlNode)) {
            return this.appendChildNode(container, 'Html', rawHtmlNode.textContent);
        }
        else if (DomUtil.isProcessingInstruction(rawHtmlNode)) {
            return this.appendChildNode(container, 'HtmlBlock', '<?' + rawHtmlNode.data + '?>');
        }
        else {
            return null;
        }
    };
    RawHtmlConversion.prototype.trueForAllChildren = function (predicate) {
        var domNode = this.rawHtmlNode;
        var walker = new DomWalker(this.rawHtmlNode);
        walker.next();
        var result = true;
        while (result && (domNode = walker.next().domNode) !== this.rawHtmlNode) {
            result = predicate(domNode);
        }
        return result;
    };
    RawHtmlConversion.prototype.appendChildNode = function (container, nodeName, literal) {
        if (literal === void 0) { literal = null; }
        var node = new commonmark.Node(nodeName);
        node.literal = literal;
        container.appendChild(node);
        return node;
    };
    return RawHtmlConversion;
})(NodeConversion);
module.exports = Converter;
//# sourceMappingURL=Converter.js.map